{% extends "layouts/content.html" %}


{% block breadcrumbs %}
{{ block.super }}
<li><a href="{% url lava_scheduler_app.views.index %}">Scheduler</a></li>
{% endblock %}


{% block content %}
<p>Hello World from LAVA Scheduler</p>
{% endblock %}
